package co.barkbuddy;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONException;
import org.json.JSONObject;

import com.actionbarsherlock.app.SherlockActivity;
import com.google.gson.Gson;
import com.parse.LogInCallback;
import com.parse.Parse;
import com.parse.ParseException;
import com.parse.ParseFile;
import com.parse.ParseUser;
import com.parse.SignUpCallback;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.graphics.drawable.BitmapDrawable;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.Display;
import android.view.Gravity;
import android.view.View;
import android.webkit.WebChromeClient;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.PopupWindow;
import android.widget.Toast;

public class LoginActivity extends SherlockActivity {
	
	private ParseUser currentUser;	

	public static ProgressDialog pd;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.login);

		pd = new ProgressDialog(this);
		pd.setIndeterminate(true);
		pd.setMessage("Logging in...");
		
		currentUser = ParseUser.getCurrentUser();
		if (currentUser != null) {
		  login(null);
		}
	}

	public void onResume() {
		super.onResume();
		
		// extract the OAUTH access token if it exists
		Uri uri = this.getIntent().getData();
		if (uri != null) {
			String code = null;

			// Check if there is a parameter called "code"
			if ((code = uri.getQueryParameter("code")) != null) {
				
				try {					
					// Call Foursquare again to get the access token
					JSONObject tokenJson = executeHttpGet("https://foursquare.com/oauth2/access_token"
							+ "?client_id="
							+ BarkBuddyApplication.FOURSQUARE_CLIENT_ID
							+ "&client_secret="
							+ BarkBuddyApplication.FOURSQUARE_CLIENT_SECRET
							+ "&grant_type=authorization_code"
							+ "&redirect_uri=" + BarkBuddyApplication.CALLBACK_URL + "&code=" + code);

					final String token = tokenJson.getString("access_token");

					// Get userdata of myself
					JSONObject responseJson = loginToFoursquare(token);

					// Get return code
					int returnCode = Integer.parseInt(responseJson.getJSONObject(
							"meta").getString("code"));
					
					// 200 = OK
					if (returnCode == 200) {

						final JSONObject userJson = responseJson.getJSONObject("response").getJSONObject("user");						
						final String email = userJson.getJSONObject("contact").getString("email");
						
						login(email, token, userJson);						
						
					} else {
						Toast.makeText(this, "Wrong return code: " + code,
								Toast.LENGTH_SHORT).show();
					}
				} catch (Exception exp) {
					Log.e("LoginTest", "Login to Foursquare failed", exp);
				    Toast.makeText(LoginActivity.this, "Login to Foursquare failed", 2000).show();
				} 
				
			} else if ((code = uri.getQueryParameter("error")) != null) {
				Toast.makeText(this, code, Toast.LENGTH_SHORT).show();
			} else {
				Toast.makeText(this, "Unknown login error", Toast.LENGTH_SHORT)
						.show();
			}
		}
	}

	private void login(final String username, final String password, final JSONObject userJson) {

		if (pd != null)
			pd.show();

		ParseUser.logInInBackground(username, password, new LogInCallback() {
			  public void done(ParseUser user, ParseException e) {
			    if (user != null) {
			    	login(null);
			    } else {
			      if (e.getCode() == ParseException.USERNAME_MISSING) {
					try {
						signUpUsingFoursquare(password, userJson, username);
					} catch (MalformedURLException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					} catch (JSONException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					} catch (IOException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
			      }
			      
			    }
			  }
			});
	}

	private void signUpUsingFoursquare(String token,
			JSONObject userJson, String email) throws JSONException,
			MalformedURLException, IOException {

		if (pd != null)
			pd.show();

		String name = userJson.getString("firstName");
		String home = userJson.getString("homeCity");
		String photoUrl = userJson.getString("photo");						
		
		currentUser = new ParseUser();
		currentUser.setUsername(email);
		currentUser.setEmail(email);
		currentUser.setPassword(token);
		currentUser.put("foursquareToken", token);
		currentUser.put("name", name);
		currentUser.put("home", home);
		
		URL u = new URL(photoUrl);
		URLConnection conn = u.openConnection();
		ByteArrayOutputStream bytes = new ByteArrayOutputStream();
		int in;
		while ((in = conn.getInputStream().read()) != -1)
			bytes.write((byte)in);
		byte[] photoData = bytes.toByteArray();
		
		ParseFile photo = new ParseFile("photo.jpg", photoData);
		photo.saveInBackground();
		currentUser.put("photo", photo);
		
		currentUser.signUpInBackground(new SignUpCallback() {
			  public void done(ParseException e) {
				    if (e == null) {
				    	login(null);
				    } else {
				      Toast.makeText(LoginActivity.this, "Parse error.", 2000).show();
				      Log.e("Parse Error", e.toString());
				    }
				  }
				});
	}

	private JSONObject loginToFoursquare(String token) throws Exception {
		return executeHttpGet("https://api.foursquare.com/v2/"
				+ "users/self" + "?oauth_token=" + token);
	}
	
	// Calls a URI and returns the answer as a JSON object
	private JSONObject executeHttpGet(String uri) throws Exception {
		HttpGet req = new HttpGet(uri);

		HttpClient client = new DefaultHttpClient();
		HttpResponse resLogin = client.execute(req);
		BufferedReader r = new BufferedReader(new InputStreamReader(resLogin
				.getEntity().getContent()));
		StringBuilder sb = new StringBuilder();
		String s = null;
		while ((s = r.readLine()) != null) {
			sb.append(s);
		}

		return new JSONObject(sb.toString());
	}

	public void login(View button) {
		Intent launchMain = new Intent(this, MainActivity.class);
		launchMain.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
		startActivity(launchMain);
		finish();
	}

	public void loginUsingFoursquare(View button) {
		// Call the webbrowser with the Foursquare OAuth login URL
		 View layout = getLayoutInflater().inflate(R.layout.oauth_popup, null,
		 false);
		WebView webView = new WebView(this);// (WebView)layout.findViewById(R.id.oauthWeb);
		webView.getSettings().setJavaScriptEnabled(true);
		webView.setScrollBarStyle(WebView.SCROLLBARS_OUTSIDE_OVERLAY);
		webView.loadUrl(Uri.parse(
				"https://foursquare.com/oauth2/authenticate" + "?client_id="
						+ BarkBuddyApplication.FOURSQUARE_CLIENT_ID + "&response_type=code" + "&redirect_uri="
						+ BarkBuddyApplication.CALLBACK_URL).toString() + "&display=touch");

		final Activity activity = this;
		final ProgressDialog progressDialog = new ProgressDialog(activity);
		progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
		progressDialog.setCancelable(false);

		webView.setWebChromeClient(new WebChromeClient() {
			@Override
			public void onProgressChanged(WebView view, int progress) {
				progressDialog.show();
				progressDialog.setProgress(0);
				progressDialog.setMessage("Loading Foursquare...");
				activity.setProgress(progress * 1000);

				progressDialog.incrementProgressBy(progress);

				if (progress == 100 && progressDialog.isShowing())
					progressDialog.dismiss();
			}
		});
//
//		CookieSyncManager.createInstance(this);
//		CookieSyncManager.getInstance().sync();
//		// CookieManager cookieManager = CookieManager.getInstance();
//		// String cookie = cookieManager.getCookie("https://foursquare.com");
		setContentView(webView);
//
		webView.setWebViewClient(new WebViewClient() {
			@Override
			public boolean shouldOverrideUrlLoading(WebView view, String url) {
				if (url.startsWith("bark"))
					return false;
				view.loadUrl(url);
				return true;
			}
		});
//
//		 Display display = getWindowManager().getDefaultDisplay();
//		 int width = display.getWidth();
//		 int height = display.getHeight();
//		 PopupWindow pw = new PopupWindow(layout, width, height, true);
//		 pw.setBackgroundDrawable(new BitmapDrawable());
//		 pw.showAtLocation(layout, Gravity.CENTER, 0, 0);

		if (pd != null)
			pd.show();
//		
//		Intent intent = new Intent(Intent.ACTION_VIEW);
//		intent.setFlags(Intent.FLAG_ACTIVITY_NO_HISTORY|Intent.FLAG_ACTIVITY_PREVIOUS_IS_TOP);
//		intent.setData(Uri.parse("https://foursquare.com/oauth2/authenticate"
//				+ "?client_id=" + BarkBuddyApplication.FOURSQUARE_CLIENT_ID + "&response_type=code"
//				+ "&redirect_uri=" + BarkBuddyApplication.CALLBACK_URL + "&display=touch"));
//		startActivity(intent);
	}
}
