package co.barkbuddy.location;

import java.util.Collections;
import java.util.List;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.PorterDuff.Mode;
import android.graphics.PorterDuffXfermode;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.util.Log;
import android.view.MotionEvent;
import android.widget.Button;
import android.widget.Toast;

import co.barkbuddy.R;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.MyLocationOverlay;
import com.google.android.maps.Overlay;
import com.google.android.maps.OverlayItem;

public class DoubleTapMapView extends MapView {
	
	private static final long TIME_INITIAL = -1;
    private static final long TIME_DOUBLE_TAP = 250;

    private long lastTouchTime = TIME_INITIAL;
    
    private MapItemizedOverlay avatarOverlays;
    private List<Overlay> mapOverlays;
    private MapController controller;
    private boolean doubleTap;
    private Context context;
    int i = 0;
	
	public DoubleTapMapView(Context context, AttributeSet attrs) {		
		super(context, attrs);
		
		this.context = context;

		mapOverlays = this.getOverlays();
		controller = this.getController();
		
		doubleTap = false;
		
		Drawable drawable = this.getResources().getDrawable(R.drawable.overlay);
		avatarOverlays = new MapItemizedOverlay(drawable, context);		
		mapOverlays.add(avatarOverlays);
    }
	
	@Override
    public boolean onInterceptTouchEvent(MotionEvent ev) {
		Log.d("ONKO", "OnInterceptTouchEvent");
        if (ev.getAction() == MotionEvent.ACTION_DOWN) {
            long thisTime = System.currentTimeMillis();
            GeoPoint p = getProjection().fromPixels((int) ev.getX(), (int) ev.getY());
            if (thisTime - lastTouchTime <= TIME_DOUBLE_TAP) {
//            	getController().animateTo(p);
//	        	getController().zoomIn();
//            	doubleTap = true;
            	

    	        int x = p.getLatitudeE6() - p.getLatitudeE6() % 2000 + 1000; 
    	        int y = p.getLongitudeE6() - p.getLongitudeE6() % 3000 + 1500; 
    	        p = new GeoPoint(x, y);
    	        
            	getOverlays().add(0, new TerritoryOverlay(p));
            	
            	OverlayItem overlay = new OverlayItem(p, "Ahoj", "i: ");
            	Canvas canvas = new Canvas();
                Bitmap overlayImage = BitmapFactory.decodeResource(getResources(), R.drawable.territory_overlay);
                Bitmap mask = BitmapFactory.decodeResource(getResources(), R.drawable.territory_overlay_mask);
                
                Bitmap mainImage = BitmapFactory.decodeResource(getResources(), R.drawable.ujolubo);
                mainImage = Bitmap.createScaledBitmap(mainImage, mask.getWidth(), mask.getHeight(), false);
                Bitmap result = Bitmap.createBitmap(overlayImage.getWidth(), overlayImage.getHeight(), Bitmap.Config.ARGB_8888);
                canvas.setBitmap(result);
                Paint paint = new Paint();
                paint.setFilterBitmap(false);

                canvas.drawBitmap(mainImage, 0, 0, paint);
                paint.setXfermode(new PorterDuffXfermode(Mode.DST_IN));
                canvas.drawBitmap(mask, 0, 0, paint);
                paint.setXfermode(null);
                canvas.drawBitmap(overlayImage, 0, 0, paint);
                
                BitmapDrawable bd = new BitmapDrawable(this.getResources(), result);
                bd.setBounds(0, 0, bd.getIntrinsicWidth(), bd.getIntrinsicHeight());
                overlay.setMarker(bd);
            	
	        	addOverlayItem(overlay, true);
            	
            	
                lastTouchTime = TIME_INITIAL;
            } else {
            	// getController().animateTo(p);
            	
//            	OverlayItem overlay = new OverlayItem(p, "Ahoj", "i: ");
//            	Canvas canvas = new Canvas();
//                Bitmap overlayImage = BitmapFactory.decodeResource(getResources(), R.drawable.overlay);
//                Bitmap mask = BitmapFactory.decodeResource(getResources(), R.drawable.mask);
//                
//                Bitmap mainImage = BitmapFactory.decodeResource(getResources(), R.drawable.ujolubo);
//                mainImage = Bitmap.createScaledBitmap(mainImage, mask.getWidth(), mask.getHeight(), false);
//                Bitmap result = Bitmap.createBitmap(overlayImage.getWidth(), overlayImage.getHeight(), Bitmap.Config.ARGB_8888);
//                canvas.setBitmap(result);
//                Paint paint = new Paint();
//                paint.setFilterBitmap(false);
//
//                canvas.drawBitmap(mainImage, 0, 0, paint);
//                paint.setXfermode(new PorterDuffXfermode(Mode.DST_IN));
//                canvas.drawBitmap(mask, 0, 0, paint);
//                paint.setXfermode(null);
//                canvas.drawBitmap(overlayImage, 0, 0, paint);
//                
//                BitmapDrawable bd = new BitmapDrawable(this.getResources(), result);
//                bd.setBounds(0, 0, bd.getIntrinsicWidth(), bd.getIntrinsicHeight());
//                overlay.setMarker(bd);
//            	
//	        	addOverlayItem(overlay);
//                doubleTap = false;
                lastTouchTime = thisTime;              
            }
        }
        return super.onInterceptTouchEvent(ev);
    }
	
	int lastZoom = 0;
	
	@Override
	public void dispatchDraw(Canvas canvas) {
		int zoom = getZoomLevel();
		if (lastZoom != zoom) {
			Toast.makeText(context, "Zoom: " + zoom, 2000).show();
			lastZoom = zoom;
			if (zoom < 16)
				this.avatarOverlays.hide();
			else
				this.avatarOverlays.unhide();
		}
		super.dispatchDraw(canvas);
	}
	
	public boolean wasDoubleTap() {
		return doubleTap;
	}
	
	private void onTap(GeoPoint point) {
		controller.animateTo(point);
	}
	
	public void addOverlayItem(OverlayItem item, boolean boundCenter) {
		avatarOverlays.addOverlay(item, boundCenter);
	}
}
