package co.barkbuddy.location;

import java.util.ArrayList;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.drawable.Drawable;
import android.util.Log;

import com.WazaBe.HoloEverywhere.HoloAlertDialogBuilder;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.ItemizedOverlay;
import com.google.android.maps.MapView;
import com.google.android.maps.OverlayItem;

public class MapItemizedOverlay extends ItemizedOverlay<OverlayItem> {
	
	private ArrayList<OverlayItem> hiddenOverlays;
	private ArrayList<OverlayItem> overlays = new ArrayList<OverlayItem>();
	private Context context;
	private int tappedOverlayIndex = -1;
	private boolean hidden = false;

	public MapItemizedOverlay(Drawable marker) {
		super(marker);
	}

	public MapItemizedOverlay(Drawable marker, Context context) {
		super(boundCenterBottom(marker));
		this.context = context;
		populate();
	}
	
	public void hide() {
		if (!hidden) {
			hiddenOverlays = new ArrayList<OverlayItem>(overlays);
			clear();		
			hidden = true;
		}
	}
	
	public void unhide() {
		if (hidden) {
			overlays = new ArrayList<OverlayItem>(hiddenOverlays);
			populate();
			hidden = false;
		}
	}
	
	public void clear() {
		overlays.clear();
		populate();
	}

	public void addOverlay(OverlayItem item, boolean boundCenter) {
		Drawable marker;
		if (boundCenter)
			marker = boundCenter(item.getMarker(0));
		else
			marker = boundCenterBottom(item.getMarker(0));
		
		item.setMarker(marker);
		
		if (hidden)
			hiddenOverlays.add(item);
		else
			overlays.add(item);
		
		this.populate();
	}
	
	@Override
	public OverlayItem createItem(int i) {
		return overlays.get(i);
	}
	
	@Override
	public int size() {		
		return overlays.size();
	}
	
	@Override
	public void draw(Canvas canvas, MapView map, boolean shadow){ 
		super.draw(canvas, map, false);
	}
	
	@Override
	public boolean onTap(final GeoPoint p, final MapView mapView) {
		Log.d("ONKO", "onTap");
		boolean tapped = super.onTap(p, mapView);
	    if (tapped && tappedOverlayIndex != -1) {            
	    	OverlayItem item = overlays.get(tappedOverlayIndex);
			HoloAlertDialogBuilder dialog = new HoloAlertDialogBuilder(context);
			dialog.setTitle(item.getTitle());
			dialog.setMessage(item.getSnippet());
			dialog.show();          
	    }           
	    else {
//	        DoubleTapMapView map = (DoubleTapMapView)mapView;
//	        if (map.wasDoubleTap()) {
//	        	map.getController().animateTo(p);
//	        	map.getController().zoomIn();
//	        }
//	        else {
//	        	map.getController().animateTo(p);
//	        	map.addOverlayItem(new OverlayItem(p, "Ahoj", "i: " + tappedOverlayIndex));
//	        }
        }                   
	    return true;
	}
	
	@Override
	protected boolean onTap(int i) {	
		Log.d("DEBUG", "i: " + i);
		tappedOverlayIndex = i;
		return true;
	}
}
