package co.barkbuddy.location;

import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.Point;
import android.location.Location;
import android.util.Log;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;

public class TerritoryOverlay extends Overlay {
	GeoPoint point;
	
	public TerritoryOverlay(GeoPoint point) {
		this.point = point;
	}
	
	public void draw(Canvas canvas, MapView map, boolean shadow){ 
        
		if (point != null) {
	        int x = point.getLatitudeE6() - point.getLatitudeE6() % 2000 + 1000; 
	        int y = point.getLongitudeE6() - point.getLongitudeE6() % 3000 + 1500; 
	        
	        GeoPoint gp1 = new GeoPoint(x - 1000, y - 1500);
	        GeoPoint gp2 = new GeoPoint(x - 1000, y + 1500);
	        GeoPoint gp3 = new GeoPoint(x + 1000, y + 1500);
	        GeoPoint gp4 = new GeoPoint(x + 1000, y - 1500);
	        
	        drawBox(gp1, gp2, gp3, gp4, map, canvas);
		}
		else {
			Log.e("TerritoryOverlay", "location point is null");
		}
        super.draw(canvas, map, shadow);
	}
	
	private void drawBox(GeoPoint gp1, GeoPoint gp2, GeoPoint gp3, GeoPoint gp4, MapView map, Canvas canvas) {       
    	Paint mPaint = new Paint();
        mPaint.setDither(true);
        mPaint.setStrokeJoin(Paint.Join.MITER);
        mPaint.setStrokeCap(Paint.Cap.SQUARE);
        mPaint.setStrokeWidth(2);
        
		Path path = new Path();
        Point p = new Point();
        
        map.getProjection().toPixels(gp1, p);
        path.moveTo(p.x, p.y);
        
        map.getProjection().toPixels(gp2, p);
        path.lineTo(p.x, p.y);
        
        map.getProjection().toPixels(gp3, p);
        path.lineTo(p.x, p.y);
        
        map.getProjection().toPixels(gp4, p);
        path.lineTo(p.x, p.y);
        
        map.getProjection().toPixels(gp1, p);
        path.lineTo(p.x, p.y);
        path.setLastPoint(p.x, p.y);

        mPaint.setStyle(Paint.Style.STROKE);
        mPaint.setColor(Color.BLACK);
        mPaint.setAlpha(255);
        canvas.drawPath(path, mPaint);

        mPaint.setColor(Color.GREEN);
        mPaint.setStyle(Paint.Style.FILL);
        mPaint.setAlpha(30);
        canvas.drawPath(path, mPaint);
	}
}
